const String _envName = 'ENV_NAME';
const String _dataSource = 'DATA_SOURCE';
const String _loggerLevel = 'LOGGER_LEVEL';
const String _featureFlags = 'FEATURE_FLAGS';

enum BuildConfigKey {
  envName(_envName, isRequired: true),
  dataSource(_dataSource, isRequired: true),
  loggerLevel(_loggerLevel),
  featureFlags(_featureFlags);

  const BuildConfigKey(this.key, {this.isRequired = false});

  final String key;
  final bool isRequired;

  /// Returns true if the environment variable is defined
  ///
  /// Cannot use `const bool.hasEnvironment(key)` because it is not a
  /// constant expression. This is why we need to use a switch statement.
  bool get isDefined => switch (this) {
        BuildConfigKey.envName => const bool.hasEnvironment(_envName),
        BuildConfigKey.dataSource => const bool.hasEnvironment(_dataSource),
        BuildConfigKey.loggerLevel => const bool.hasEnvironment(_loggerLevel),
        BuildConfigKey.featureFlags => const bool.hasEnvironment(_featureFlags),
      };

  /// Returns the value of the environment variable
  ///
  /// Cannot use `const String.fromEnvironment(key)` because it is not a
  /// constant expression. This is why we need to use a switch statement.
  String get stringFromEnv => switch (this) {
        BuildConfigKey.envName => const String.fromEnvironment(_envName),
        BuildConfigKey.dataSource => const String.fromEnvironment(_dataSource),
        BuildConfigKey.loggerLevel =>
          const String.fromEnvironment(_loggerLevel),
        BuildConfigKey.featureFlags =>
          const String.fromEnvironment(_featureFlags),
      };

  /// Returns the values of the environment variables
  static String get summary {
    final summary = StringBuffer();
    for (final key in BuildConfigKey.values) {
      if (key.isRequired && !key.isDefined) {
        summary.writeln('${key.key} is not defined (required)');
        continue;
      }
      if (key.isDefined) {
        summary.writeln('${key.key}: ${key.stringFromEnv}');
        continue;
      }
    }

    return summary.toString();
  }

  /// Returns the value of the environment variable
  ///
  /// If the environment variable is not defined, returns null
  T? getOrNull<T>() {
    try {
      return get<T>();
    } catch (_) {
      return null;
    }
  }

  /// Returns the value of the environment variable
  ///
  /// If the environment variable is not defined, returns the value of [or]
  T getOr<T>(T Function() or) {
    switch (T) {
      case String:
        return _getStringOr(or as String Function()) as T;
      case int:
        return _getIntOr(or as int Function()) as T;
      case double:
        return _getDoubleOr(or as double Function()) as T;
      case bool:
        return _getBool() as T;
      default:
        throw Exception('Type $T is not supported');
    }
  }

  /// Returns the value of the environment variable
  ///
  /// Throws an exception if the environment variable is not defined
  T get<T>() {
    switch (T) {
      case String:
        return _getString() as T;
      case int:
        return _getInt() as T;
      case double:
        return _getDouble() as T;
      case bool:
        return _getBool() as T;
      default:
        throw Exception('Type $T is not supported');
    }
  }

  String _getString() {
    final value = stringFromEnv;
    if (value.isEmpty) {
      throw Exception('$key is not defined');
    }

    return value;
  }

  int _getInt() {
    // Use int.tryParse instead int.fromEnvironment because we want to
    // support 0 as a value too.
    final value = int.tryParse(stringFromEnv);
    if (value == null) {
      throw Exception('$key is not defined');
    }

    return value;
  }

  double _getDouble() {
    final value = double.tryParse(stringFromEnv);
    if (value == null) {
      throw Exception('$key is not defined');
    }

    return value;
  }

  bool _getBool() {
    final value = stringFromEnv.toLowerCase();

    return value == 'true' || value == '1';
  }

  double _getDoubleOr(double Function() or) {
    try {
      return _getDouble();
    } catch (_) {
      return or();
    }
  }

  String _getStringOr(String Function() or) {
    try {
      return _getString();
    } catch (_) {
      return or();
    }
  }

  int _getIntOr(int Function() or) {
    try {
      return _getInt();
    } catch (_) {
      return or();
    }
  }
}
