import 'package:logging/logging.dart';
import 'package:starting_template/core/logger/app_logger_plugin.dart';
import 'package:starting_template/core/logger/log_level.dart';
import 'package:starting_template/core/mixins/debuggable.dart';

class AppLogger with Debuggable {
  /// Creates a top-level [AppLogger].
  factory AppLogger([String namespace = rootNamespace]) {
    if (!namespace.startsWith(rootNamespace)) {
      namespace = '$rootNamespace.$namespace';
    }

    return AppLogger.activeLoggers[namespace] ??= AppLogger._(namespace);
  }

  AppLogger._(String namespace) : _logger = Logger(namespace);

  /// The root namespace for all [AppLogger]s.
  static const rootNamespace = 'StartingTemplate';

  static final Map<String, AppLogger> activeLoggers = {};

  String get namespace => _logger.fullName;

  final Logger _logger;

  AppLogger createChild(String name) {
    assert(name.isNotEmpty, 'Name should not be empty');

    return AppLogger('$namespace.$name');
  }

  Level get level => _logger.level;

  set level(Level level) {
    hierarchicalLoggingEnabled = true;
    _logger.level = level;
  }

  void addPlugin(AppLoggerPlugin plugin) {
    _logger.onRecord.listen(plugin.handleLogRecord);
  }

  /// Logs a [message] at the given [level].
  void log(
    Level level,
    String message, [
    Object? error,
    StackTrace? stackTrace,
  ]) {
    _logger.log(level, message, error, stackTrace);
  }

  /// Logs a message with level [LogLevel.verbose].
  void verbose(String message, [Object? error, StackTrace? stackTrace]) {
    log(LogLevel.verbose, message, error, stackTrace);
  }

  /// Logs a message with level [LogLevel.debug].
  void debug(String message, [Object? error, StackTrace? stackTrace]) {
    log(LogLevel.debug, message, error, stackTrace);
  }

  /// Logs a message with level [LogLevel.info].
  void info(String message, [Object? error, StackTrace? stackTrace]) {
    log(LogLevel.info, message, error, stackTrace);
  }

  /// Logs a message with level [LogLevel.warn].
  void warn(String message, [Object? error, StackTrace? stackTrace]) {
    log(LogLevel.warn, message, error, stackTrace);
  }

  /// Logs a message with level [LogLevel.error].
  void error(String message, [Object? error, StackTrace? stackTrace]) {
    log(LogLevel.error, message, error, stackTrace);
  }

  @override
  String get debugName => 'AppLogger';
}
